/*
 *  Copyright (c) 2010-2011, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information:
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *
*/

/* This file has primarily those functions that were earlier generated by XDC
 * TO DO: Clean up to ensure it is readable */

#include <xdc/std.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
 
#ifndef _LOCAL_CORE_a8host_
  #include <xdc/runtime/Log.h>
  #include <xdc/runtime/Assert.h>
  #include <xdc/runtime/Diags.h>
#else
  /* Use Slog at A8 side */
  #if (USE_SLOG_PRINT == 1)
    #include <Log.h>
    #include <LoggerSys.h>

    /* Every module needs to define unique module id */
    #ifndef SLOG_DOMXRPC_Module__MID
      #define SLOG_DOMXRPC_Module__MID (SLOG_DOMX_Module__MID)
    #endif
    #undef Module__MID
    #define Module__MID SLOG_DOMXRPC_Module__MID
  #else
    #include <xdc/runtime/Log.h>
    #include <xdc/runtime/Assert.h>
    #include <xdc/runtime/Diags.h>
  #endif /* End Of #if (USE_SLOG_PRINT == 1) */
#endif /* End of #else */
#include <xdc/runtime/Core.h>

#include <ti/omx/interfaces/openMaxv11/OMX_Core.h>
#include <ti/omx/interfaces/openMaxv11/interfaces_openmaxv11_internal.h>
#include "domx_internal.h"
#include "domx_internalcfg.h"
#include "OmxProxy__epilogue.h"
#include <timm_osal_interfaces.h>
#include "domx_util.h"


/* Symbol defined by XDC generated file */
#ifdef _LOCAL_CORE_c6xdsp_
extern far xdc_Int xdc_runtime_knl_Thread_Object__PARAMS__C;
#else
extern xdc_Int xdc_runtime_knl_Thread_Object__PARAMS__C;
#endif

void OmxRpc_Params_init (OmxRpc_Params *prms)
{
  DOMX_UTL_TRACE_FUNCTION_ENTRY_LEVEL1 ();
  xdc_runtime_Core_assignParams__I (prms,
                                    (xdc_Ptr)
                                    (&xdc_runtime_knl_Thread_Object__PARAMS__C),
                                    sizeof (OmxRpc_Params),
                                    sizeof (xdc_runtime_IInstance_Params));
  DOMX_UTL_TRACE_FUNCTION_EXIT_LEVEL1 (OMX_ErrorNone);
}

/* Object__create__S */
xdc_Ptr OmxRpc_object_create (xdc_Ptr oa, xdc_SizeT osz,
                              const OmxRpc_Params *aa,
                              xdc_SizeT psz, xdc_runtime_Error_Block *eb)
{
  OmxRpc_Object *obj;

  DOMX_UTL_TRACE_FUNCTION_ENTRY_LEVEL1 ();

  /* common instance initialization */
  obj = TIMM_OSAL_MallocExtn (sizeof (OmxRpc_Object),
                              TIMM_OSAL_TRUE, 0,
                              TIMMOSAL_MEM_SEGMENT_EXT, NULL);
  if (obj == NULL)
  {
    return NULL;
  }

  /* module-specific initialization */
  OmxRpc_Instance_init (obj, aa, eb);

  DOMX_UTL_TRACE_FUNCTION_EXIT_LEVEL1 (OMX_ErrorNone);
  return obj;
}

/* create */
OmxRpc_Handle OmxRpc_create (const OmxRpc_Params *prms,
                             xdc_runtime_Error_Block *eb)
{
  return (OmxRpc_Handle) OmxRpc_object_create (NULL, 0,
                                               prms,
                                               (xdc_SizeT)
                                               sizeof (OmxRpc_Params), eb);
}

extern Void OmxRpc_Instance_finalize (OmxRpc_Object *obj, Int status);

/* delete */
void OmxRpc_delete (OmxRpc_Handle *instp)
{
  DOMX_UTL_TRACE_FUNCTION_ENTRY_LEVEL1 ();
  OmxRpc_Instance_finalize ((OmxRpc_Object *) (*instp), 0 /* dont care
                                                             param */ );
  TIMM_OSAL_Free (*instp);
  *((OmxRpc_Handle *) instp) = NULL;
  DOMX_UTL_TRACE_FUNCTION_EXIT_LEVEL1 (OMX_ErrorNone);
}

/* Nothing beyond this point */
